@@ -3,17 +3,15 @@
 #
 # Test statistics from Berman (1986)
 #
-#  $Revision: 1.14 $  $Date: 2014/02/07 09:09:38 $
+#  $Revision: 1.21 $  $Date: 2018/01/17 08:46:51 $
 #
 #
 
-# ---------------------------
-
-bermantest <- function(...) {
-  UseMethod("bermantest")
+berman.test <- function(...) {
+  UseMethod("berman.test")
 }
 
-bermantest.ppp <-
+berman.test.ppp <-
   function(X, covariate,
            which=c("Z1", "Z2"),
            alternative=c("two.sided", "less", "greater"),
@@ -24,14 +22,14 @@ bermantest.ppp <-
     which <- match.arg(which)
     alternative <- match.arg(alternative)
 
-    do.call("bermantestEngine",
+    do.call(bermantestEngine,
             resolve.defaults(list(ppm(X), covariate, which, alternative),
                              list(...),
                              list(modelname="CSR",
                                   covname=covname, dataname=Xname)))
 }
 
-bermantest.ppm <- function(model, covariate,
+berman.test.ppm <- function(model, covariate,
                            which=c("Z1", "Z2"),
                            alternative=c("two.sided", "less", "greater"),
                            ...) {
@@ -43,7 +41,7 @@ bermantest.ppm <- function(model, covariate,
   alternative <- match.arg(alternative)
   if(is.poisson(model) && is.stationary(model))
     modelname <- "CSR"
-  do.call("bermantestEngine",
+  do.call(bermantestEngine,
           resolve.defaults(list(model, covariate, which, alternative),
                            list(...),
                            list(modelname=modelname,
@@ -51,7 +49,7 @@ bermantest.ppm <- function(model, covariate,
                                 dataname=model$Qname)))
 }
 
-bermantest.lpp <-
+berman.test.lpp <-
   function(X, covariate,
            which=c("Z1", "Z2"),
            alternative=c("two.sided", "less", "greater"),
@@ -62,14 +60,14 @@ bermantest.lpp <-
     which <- match.arg(which)
     alternative <- match.arg(alternative)
 
-    do.call("bermantestEngine",
+    do.call(bermantestEngine,
             resolve.defaults(list(lppm(X), covariate, which, alternative),
                              list(...),
                              list(modelname="CSR",
                                   covname=covname, dataname=Xname)))
 }
 
-bermantest.lppm <- function(model, covariate,
+berman.test.lppm <- function(model, covariate,
                            which=c("Z1", "Z2"),
                            alternative=c("two.sided", "less", "greater"),
                            ...) {
@@ -81,7 +79,7 @@ bermantest.lppm <- function(model, covariate,
   alternative <- match.arg(alternative)
   if(is.poisson(model) && is.stationary(model))
     modelname <- "CSR"
-  do.call("bermantestEngine",
+  do.call(bermantestEngine,
           resolve.defaults(list(model, covariate, which, alternative),
                            list(...),
                            list(modelname=modelname,
@@ -109,24 +107,45 @@ bermantestEngine <- function(model, covariate,
   if(!is.poisson(model))
     stop("Only implemented for Poisson point process models")
 
-  # ........... first assemble data ...............
+  #'  compute required data 
   fram <- spatialCDFframe(model, covariate, ...,
-                        modelname=modelname,
-                        covname=covname,
-                        dataname=dataname)
+                          modelname=modelname,
+                          covname=covname,
+                          dataname=dataname)
+  #'  evaluate berman test statistic 
+  result <- bermantestCalc(fram, which=which, alternative=alternative)
+
+  return(result)
+}
+
+bermantestCalc <- function(fram,
+                           which=c("Z1", "Z2"),
+                           alternative=c("two.sided", "less", "greater"),
+                           ...) {
+
+  which <- match.arg(which)
+  alternative <- match.arg(alternative)
+
+  verifyclass(fram, "spatialCDFframe")
   fvalues <- fram$values
   info    <- fram$info
-  # values of covariate at data points
+  
+  ## values of covariate at data points
   ZX <- fvalues$ZX
-  # transformed to Unif[0,1] under H0
+  ## transformed to Unif[0,1] under H0
   U  <- fvalues$U
-  # values of covariate at pixels
+  ## values of covariate at pixels
   Zvalues <- fvalues$Zvalues
-  # corresponding pixel areas/weights
+  ## corresponding pixel areas/weights
   weights <- fvalues$weights
-  # intensity of model
+  ## intensity of model
   lambda  <- fvalues$lambda
 
+  ## names 
+  modelname <- info$modelname
+  dataname  <- info$dataname
+  covname   <- info$covname
+  
   switch(which,
          Z1={
            #......... Berman Z1 statistic .....................
@@ -199,8 +218,9 @@ bermantestEngine <- function(model, covariate,
 }
 
 plot.bermantest <-
-  function(x, ..., lwd=par("lwd"), col=par("col"), lty=par("lty"),
-           lwd0=lwd, col0=col, lty0=lty)
+  function(x, ...,
+           lwd=par("lwd"), col=par("col"), lty=par("lty"),
+           lwd0=lwd, col0=2, lty0=2)
 {
   fram <- x$fram
   if(!is.null(fram)) {
@@ -225,7 +245,7 @@ plot.bermantest <-
                            sQuote(info$covname)),
                      paste("Z1 statistic =", signif(x$statistic, 4)),
                      paste("p-value=", signif(x$p.value, 4)))
-           do.call("plot.default",
+           do.call(plot.default,
                    resolve.defaults(
                                     list(x=xxx, y=yyy, type="l"),
                                     list(...),
@@ -249,11 +269,12 @@ plot.bermantest <-
                            sQuote(info$covname)),
                      paste("Z2 statistic =", signif(x$statistic, 4)),
                      paste("p-value=", signif(x$p.value, 4)))
-           do.call("plot.ecdf",
+           do.call(plot.ecdf,
                    resolve.defaults(
                                     list(cdfU),
                                     list(...),
                                     list(do.points=FALSE, asp=1),
+                                    list(xlim=c(0,1), ylim=c(0,1)),
                                     list(lwd=lwd, col=col, lty=lty),
                                     list(xlab="U", ylab="relative frequency"),
                                     list(main=main)))
