@@ -1,16 +1,30 @@
-#
-# deldir.R
-#
-# Interface to deldir package
-#
-#  $Revision: 1.28 $ $Date: 2017/06/05 10:31:58 $
-#
+#'
+#' deldir.R
+#'
+#' Interface to deldir package
+#'
+#'  $Revision: 1.30 $ $Date: 2018/09/28 05:12:32 $
+#'
 
-.spst.triEnv <- new.env()
+#' ..............................................
+#' Internal options 
+#'      deldir suggests spatstat (!!!)
+#'      so we must save options here, not in spatstat.options
+  .spst.triEnv <- new.env()
+  assign("use.trigraf",  TRUE, envir=.spst.triEnv)
+  assign("use.trigrafS", TRUE, envir=.spst.triEnv)
+  assign("debug.delaunay", FALSE, envir=.spst.triEnv)
+#' for testing purposes only
+  spatstat.deldir.setopt <- function(use.trigrafS=TRUE,
+                                     use.trigraf=TRUE,
+                                     debug.delaunay=FALSE) {
+    assign("use.trigrafS", use.trigrafS, envir=.spst.triEnv)
+    assign("use.trigraf",  use.trigraf, envir=.spst.triEnv)
+    assign("debug.delaunay", debug.delaunay, envir=.spst.triEnv)
+    return(invisible(NULL))
+  }
+#'..............................................
 
-assign("use.trigraf",  TRUE, envir=.spst.triEnv)
-assign("use.trigrafS", TRUE, envir=.spst.triEnv)
-assign("debug.delaunay", FALSE, envir=.spst.triEnv)
 
 dirichlet <- local({
 
@@ -102,7 +116,7 @@ delaunay <- function(X) {
     for(i in seq_len(nX)) {
       # find all Delaunay neighbours of i 
       jj <- c(b[a==i], a[b==i])
-      jj <- sort(unique(jj))
+      jj <- sortunique(jj)
       # select those with a higher index than i
       jj <- jj[jj > i]
       # find pairs of neighbours which are Delaunay neighbours
@@ -164,7 +178,7 @@ delaunay <- function(X) {
       if(debug.delaunay)
         cat(paste("\tInvestigating triangles", commasep(sustri), "\n"))
       # list all vertices associated with the suspect triangles
-      susvert <- sort(unique(as.vector(tlist[sustri, ])))
+      susvert <- sortunique(as.vector(tlist[sustri, ]))
       if(debug.delaunay)
         cat(paste("\tInvestigating vertices", commasep(susvert), "\n"))
       xsusvert <- x[susvert]
