@@ -6,7 +6,7 @@
 #  generic functions
 #  and methods for owin, psp, ppp
 #
-#  $Revision: 1.25 $   $Date: 2013/12/15 11:32:12 $
+#  $Revision: 1.31 $   $Date: 2018/05/13 03:58:54 $
 #
 
 # ............ generic  ............................
@@ -75,7 +75,7 @@ erosion.owin <-
     # compute polygonal region using polyclip package
     pnew <- polyclip::polyoffset(w$bdry, -r, jointype="round")
     # ensure correct polarity
-    totarea <- sum(unlist(lapply(pnew, area.xypolygon)))
+    totarea <- sum(unlist(lapply(pnew, Area.xypolygon)))
     if(totarea < 0)
       pnew <- lapply(pnew, reverse.xypolygon)
     if(shrink.frame) {
@@ -154,7 +154,7 @@ dilation.owin <-
     # compute polygonal region using polyclip package
     pnew <- polyclip::polyoffset(w$bdry, r, jointype="round")
     # ensure correct polarity
-    totarea <- sum(unlist(lapply(pnew, area.xypolygon)))
+    totarea <- sum(unlist(lapply(pnew, Area.xypolygon)))
     if(totarea < 0)
       pnew <- lapply(pnew, reverse.xypolygon)
     # determine bounding frame, convert to owin
@@ -175,7 +175,8 @@ closing.owin <- function(w, r, ..., polygonal=NULL) {
   if(is.empty(wplus))
     return(wplus)
   wclose <- erosion.owin(wplus, r, strict=TRUE)
-  wclose <- rebound.owin(wclose, as.rectangle(w))
+  b <- as.rectangle(w)
+  wclose <- rebound.owin(wclose[b], b)
   return(wclose)
 }
 
@@ -187,7 +188,8 @@ opening.owin <- function(w, r, ..., polygonal=NULL) {
   if(is.empty(wminus))
     return(wminus)
   wopen <- dilation.owin(wminus, r, tight=FALSE)
-  wopen <- rebound.owin(wopen, as.rectangle(w))
+  b <- as.rectangle(w)
+  wopen <- rebound.owin(wopen[b], b)
   return(wopen)
 }
 
@@ -231,7 +233,7 @@ dilation.psp <- function(w, r, ..., polygonal=TRUE, tight=TRUE) {
     # old code for polygonal case
     ends   <- x$ends
     angles <- angles.psp(x, directed=TRUE)
-    lengths <- lengths.psp(x)
+#    lengths <- lengths.psp(x)
     out <- NULL
     # dilate individual segments
     halfcircle <- seq(from=0, to=pi, length.out=128)[-c(1,128)]
@@ -267,7 +269,7 @@ dilation.psp <- function(w, r, ..., polygonal=TRUE, tight=TRUE) {
     pnew <- polyclip::polylineoffset(plines, r,
                                      jointype="round", endtype="openround")
     # ensure correct polarity
-    totarea <- sum(unlist(lapply(pnew, area.xypolygon)))
+    totarea <- sum(unlist(lapply(pnew, Area.xypolygon)))
     if(totarea < 0)
       pnew <- lapply(pnew, reverse.xypolygon)
     # convert to owin object
@@ -313,23 +315,27 @@ dilation.ppp <- function(w, r, ..., polygonal=TRUE, tight=TRUE) {
   
   # bounding frame
   bb <- if(tight) boundingbox(x) else as.rectangle(x)
-    newbox <- grow.rectangle(bb, r)
+  releps <- 1e-6
+  newbox <- grow.rectangle(bb, r * (1+releps))
 
   # compute dilation
   if(!polygonal) {
     # compute pixel approximation
-    x <- rebound.ppp(x, newbox)
+    Window(x) <- newbox
     distant <- distmap(x, ...)
     dil <- levelset(distant, r, "<=")
     return(dil)
   } else {
     # compute polygonal approximation
     # generate discs
-    out <- NULL
-    for(i in seq_len(x$n)) {
-      balli <- disc(r, c(x$x[i], x$y[i]))
-      out <- union.owin(out, balli)
-    }
+    coo <- coords(x)
+    nn <- npoints(x)
+    balls <- vector(mode="list", length=nn)
+    ball0 <- disc(r, c(0,0), ...)
+    for(i in seq_len(nn))
+      balls[[i]] <- shift(ball0, vec=coo[i,])
+    class(balls) <- c("solist", class(balls))
+    out <- union.owin(balls)
     return(out)
   }
 }
@@ -362,17 +368,23 @@ opening.ppp <- function(w, r, ...) {
 
 # ............ utilities ............................
 
-validradius <- function(r, caller="morphological operator") {
-  rname <- short.deparse(substitute(r))
+validradius <- local({
+
+  validradius <- function(r, caller="morphological operator") {
+  #  rname <- short.deparse(substitute(r))
+    if(!is.numeric(r) || length(r) != 1)
+      groan("radius r must be a single number", caller)
+    if(r < 0)
+      groan("radius r must be nonnegative", caller)
+    return(TRUE)
+  }
+
   groan <- function(whinge, caller) {
     stop(paste("for", paste(caller, ",", sep=""), whinge), call.=FALSE)
   }
-  if(!is.numeric(r) || length(r) != 1)
-    groan("radius r must be a single number", caller)
-  if(r < 0)
-    groan("radius r must be nonnegative", caller)
-  return(TRUE)
-}
+
+  validradius
+})
   
 idorempty <- function(w, r, caller="morphological operator") {
   validradius(r, caller)
