@@ -1,7 +1,7 @@
 #
 #   plot.im.R
 #
-#  $Revision: 1.120 $   $Date: 2017/10/03 02:01:01 $
+#  $Revision: 1.125 $   $Date: 2018/10/18 09:36:06 $
 #
 #  Plotting code for pixel images
 #
@@ -133,10 +133,31 @@ plot.im <- local({
     return(y)
   }
 
-  Ticks <- function(usr, log=FALSE, nint=NULL, ...) {
-    #' same as axisTicks but accepts nint=NULL as if it were missing
-    if(!is.null(nint)) return(axisTicks(usr=usr, log=log, nint=nint, ...))
-    return(axisTicks(usr=usr, log=log, ...))
+  Ticks <- function(usr, log=FALSE, nint=NULL, ..., clip=TRUE) {
+    #' modification of grDevices::axisTicks
+    #'      constrains ticks to be inside the specified range if clip=TRUE
+    #'      accepts nint=NULL as if it were missing
+    z <- if(is.null(nint)) axisTicks(usr=usr, log=log, ...) else
+         axisTicks(usr=usr, log=log, nint=nint, ...) 
+    if(clip) {
+      zlimits <- if(log) 10^usr else usr
+      z <- z[inside.range(z, zlimits)]
+    }
+    return(z)
+  }
+
+  numericalRange <- function(x, zlim=NULL) {
+    xr <- suppressWarnings(range(x, finite=TRUE))
+    if(!all(is.finite(xr)))
+      warning("All pixel values are NA", call.=FALSE)
+    if(!is.null(zlim)) 
+      xr <- suppressWarnings(range(xr, zlim, finite=TRUE))
+    if(!all(is.finite(xr))) {
+      warning("Cannot determine range of values for colour map",
+              call.=FALSE)
+      xr <- c(0,0)
+    }
+    return(xr)
   }
   
   # main function
@@ -144,6 +165,7 @@ plot.im <- local({
                      main, 
                      add=FALSE, clipwin=NULL,
                      col=NULL, valuesAreColours=NULL, log=FALSE,
+                     ncolours=256, gamma=1, 
                      ribbon=show.all, show.all=!add,
                      ribside=c("right", "left", "bottom", "top"),
                      ribsep=0.15, ribwid=0.05, ribn=1024,
@@ -299,8 +321,7 @@ plot.im <- local({
 
     switch(xtype,
            real    = {
-             vrange <- range(x, finite=TRUE)
-             vrange <- range(zlim, vrange)
+             vrange <- numericalRange(x, zlim)
              if(!is.null(colmap)) {
                # explicit colour map
                s <- summary(colmap)
@@ -333,8 +354,7 @@ plot.im <- local({
              values <- as.vector(x$v)
              values <- values[!is.na(values)]
              uv <- unique(values)
-             vrange <- range(uv, finite=TRUE)
-             vrange <- range(zlim, vrange)
+             vrange <- numericalRange(uv, zlim)
              nvalues <- length(uv)
              trivial <- (nvalues < 2)
              if(!trivial){
@@ -433,8 +453,9 @@ plot.im <- local({
       colourinfo <- list(breaks=imagebreaks, col=col)
     } else if(!is.null(colfun)) {
       ## Function colfun(n)
-      colourinfo <- if(is.null(imagebreaks)) list(col=colfun(256)) else
-                    list(breaks=imagebreaks, col=colfun(length(imagebreaks)-1))
+      colourinfo <-
+        if(is.null(imagebreaks)) list(col=colfun(ncolours)) else
+        list(breaks=imagebreaks, col=colfun(length(imagebreaks) - 1L))
     } else if(col.given) {
       ## Colour values
       if(inherits(try(col2rgb(col), silent=TRUE), "try-error"))
@@ -469,7 +490,7 @@ plot.im <- local({
              real= {
                if(!is.null(i.bks)) {
                  colourmap(col=i.col, breaks=i.bks)
-               } else colourmap(col=i.col, range=vrange)
+               } else colourmap(col=i.col, range=vrange, gamma=gamma)
              },
              logical={
                colourmap(col=i.col, inputs=c(FALSE,TRUE))
@@ -480,6 +501,11 @@ plot.im <- local({
              },
              NULL)
 
+    ## gamma correction
+    soc <- summary(output.colmap)
+    if(!is.null(gamma <- soc$gamma) && gamma != 1)
+      colourinfo$breaks <- soc$breaks
+
     ##  ........ decide whether to use rasterImage .........
     
     ## get device capabilities
@@ -520,6 +546,7 @@ plot.im <- local({
                  dotargs,
                  list(useRaster=useRaster, add=add, show.all=show.all),
                  colourinfo,
+                 list(zlim=vrange),
                  list(xlab = "", ylab = ""),
                  list(asp = 1, main = main, axes=FALSE))
 ##      if(add && show.all)
@@ -564,6 +591,7 @@ plot.im <- local({
 
     attr(output.colmap, "bbox") <- bb.all
     attr(output.colmap, "bbox.legend") <- bb.rib
+    attr(output.colmap, "side.legend") <- rib.iside
     if(!do.plot)
       return(output.colmap)
 
@@ -600,6 +628,7 @@ plot.im <- local({
                dotargs,
                list(useRaster=useRaster),
                colourinfo,
+               list(zlim=vrange),
                list(xlab = "", ylab = ""),
                list(asp = 1, main = main))
 
@@ -691,9 +720,7 @@ plot.im <- local({
                                xaxp=c(bb.rib$xrange, length(ribbonticks)))
              })
       do.call.plotfun(graphics::axis,
-                      resolve.defaults(ribargs,
-                                       axisargs, dotargs,
-                                       posargs),
+                      resolve.defaults(ribargs, axisargs, dotargs, posargs),
                       extrargs=graphicsPars("axis"))
     }
     #
