@@ -1,7 +1,7 @@
 #
 #  psp.R
 #
-#  $Revision: 1.89 $ $Date: 2017/09/13 09:52:15 $
+#  $Revision: 1.94 $ $Date: 2018/10/02 03:50:30 $
 #
 # Class "psp" of planar line segment patterns
 #
@@ -97,14 +97,15 @@ as.psp.psp <- function(x, ..., check=FALSE, fatal=TRUE) {
 }
 
 as.psp.data.frame <- function(x, ..., window=NULL, marks=NULL,
-                              check=spatstat.options("checksegments"), fatal=TRUE) {
+                              check=spatstat.options("checksegments"),
+                              fatal=TRUE) {
   window <- suppressWarnings(as.owin(window,fatal=FALSE))
   if(!is.owin(window)) {
     if(fatal) stop("Cannot interpret \"window\" as an object of class owin.\n")
     return(NULL)
   }
 
-  if(checkfields(x,"marks")) {
+  if(checkfields(x, "marks")) {
     if(is.null(marks)) marks <- x$marks
     else warning(paste("Column named \"marks\" ignored;\n",
                        "argument named \"marks\" has precedence.\n",sep=""))
@@ -115,8 +116,7 @@ as.psp.data.frame <- function(x, ..., window=NULL, marks=NULL,
     out <- psp(x$x0, x$y0, x$x1, x$y1, window=window,
                check=check)
     x <- x[-match(c("x0","y0","x1","y1"),names(x))]
-  }
-  else if(checkfields(x, c("xmid", "ymid", "length", "angle"))) {
+  } else if(checkfields(x, c("xmid", "ymid", "length", "angle"))) {
     rr <- x$length/2
     dx <- cos(x$angle) * rr
     dy <- sin(x$angle) * rr
@@ -127,26 +127,35 @@ as.psp.data.frame <- function(x, ..., window=NULL, marks=NULL,
                    window=bigbox,check=FALSE)
     out <- pattern[window]
     x <- x[-match(c("xmid","ymid","length","angle"),names(x))]
-  }
-  else if(ncol(x) >= 4) {
+  } else if(ncol(x) >= 4) {
     out <- psp(x[,1], x[,2], x[,3], x[,4], window=window,
                check=check)
     x <- x[-(1:4)]
+  } else {
+    ## data not understood
+    if(fatal) 
+      stop("Unable to interpret x as a line segment pattern.", call.=FALSE)
+    return(NULL)
   }
-  else if(fatal)
-    stop("Unable to interpret x as a line segment pattern.", call.=FALSE)
-  else out <- NULL
 
-  if(!is.null(out)) {
-    if(is.null(marks) & ncol(x) > 0) marks <- x
+  if(ncol(x) > 0) {
+    #' additional columns of mark data in 'x'
     if(is.null(marks)) {
-       out$markformat <- "none"
+      marks <- x
     } else {
-       out$marks <- marks
-       out$markformat <- if(is.data.frame(marks)) "dataframe" else "vector"
-       out <- as.psp(out,check=FALSE)
+      warning(paste("Additional columns in x were ignored",
+                    "because argument 'marks' takes precedence"),
+              call.=FALSE)
     }
   }
+
+  if(!is.null(marks)) {
+    if(identical(ncol(marks), 1L)) marks <- marks[,1L]
+    #' assign marks directly to avoid infinite recursion
+    out$marks <- marks
+    out$markformat <- markformat(marks)
+  }
+
   return(out)
 }
 
@@ -328,17 +337,32 @@ plot.psp <- function(x, ..., main, add=FALSE,
                      show.all=!add, 
                      show.window=show.all,
                      which.marks=1,
+                     style=c("colour", "width", "none"),
                      ribbon=show.all, ribsep=0.15, ribwid=0.05, ribn=1024,
                      do.plot=TRUE) {
   if(missing(main) || is.null(main))
     main <- short.deparse(substitute(x))
   verifyclass(x, "psp")
-  #
+  #'
   n <- nsegments(x)
   marx <- marks(x)
-  #
-  use.colour <- !is.null(marx) && (n != 0)
-  do.ribbon <- identical(ribbon, TRUE) && use.colour 
+  #'
+  style <- match.arg(style)
+  use.marks <- !is.null(marx) && (n != 0) && (style != "none")
+  #'
+  if(use.marks && style == "width") {
+    #' plot marks as line width
+    #' temporary cheat using plot.linfun
+    L <- linnet(endpoints.psp(x), edges=cbind(2*(1:n)-1, 2*(1:n)), sparse=TRUE)
+    if(length(dim(marx))) marx <- marx[,which.marks]
+    f <- function(x,y,seg,tp, values=marx) { values[seg] }
+    g <- linfun(f, L)
+    out <- plot(g, style="width", ..., main=main, add=add,
+                show.all=show.all, show.window=show.window, do.plot=do.plot)
+    return(invisible(out))
+  }
+  #' plot marks as colours, if present
+  do.ribbon <- identical(ribbon, TRUE) && use.marks
   ##
   ## ....   initialise plot; draw observation window  ......
   owinpars <- setdiff(graphicsPars("owin"), "col")
@@ -398,7 +422,7 @@ plot.psp <- function(x, ..., main, add=FALSE,
   }
   
   # determine colours if any
-  if(!use.colour) {
+  if(!use.marks) {
     # black
     col <- colmap <- NULL
   } else {
@@ -646,14 +670,18 @@ shift.psp <- function(X, vec=c(0,0), ..., origin=NULL) {
     stopifnot(is.character(origin))
     if(!missing(vec))
       warning("Argument vec ignored; argument origin has precedence.\n")
-    origin <- pickoption("origin", origin, c(centroid="centroid",
-                                             midpoint="midpoint",
-                                             bottomleft="bottomleft"))
-    W <- as.owin(X)
-    locn <- switch(origin,
-                   centroid={ unlist(centroid.owin(W)) },
-                   midpoint={ c(mean(W$xrange), mean(W$yrange)) },
-                   bottomleft={ c(W$xrange[1], W$yrange[1]) })
+    if(is.numeric(origin)) {
+      locn <- origin
+    } else {
+      origin <- pickoption("origin", origin, c(centroid="centroid",
+                                               midpoint="midpoint",
+                                               bottomleft="bottomleft"))
+      W <- as.owin(X)
+      locn <- switch(origin,
+                     centroid={ unlist(centroid.owin(W)) },
+                     midpoint={ c(mean(W$xrange), mean(W$yrange)) },
+                     bottomleft={ c(W$xrange[1], W$yrange[1]) })
+    }
     return(shift(X, -locn))
   }
   # perform shift
@@ -699,7 +727,7 @@ identify.psp <- function(x, ..., labels=seq_len(nsegments(x)),
     stop("n should be a single integer")
   out <- integer(0)
   while(length(out) < n) {
-    xy <- locator(1)
+    xy <- spatstatLocator(1)
     # check for interrupt exit
     if(length(xy$x) == 0)
       return(out)
@@ -767,3 +795,43 @@ text.psp <- function(x, ...) {
                                    .StripNull=TRUE))
   return(invisible(NULL))
 }
+
+intensity.psp <- function(X, ..., weights=NULL) {
+  len <- lengths.psp(X)
+  a <- area(Window(X))
+  if(is.null(weights)) {
+    ## unweighted case - for efficiency
+    if(is.multitype(X)) {
+      mks <- marks(X)
+      answer <- tapply(len, mks, sum)/a
+      answer[is.na(answer)] <- 0
+      names(answer) <- levels(mks)
+    } else answer <- sum(len)/a
+    return(answer)
+  }
+  ## weighted case 
+  if(is.numeric(weights)) {
+    check.nvector(weights, nsegments(X), things="segments")
+  } else if(is.expression(weights)) {
+    # evaluate expression in data frame of coordinates and marks
+    df <- as.data.frame(X)
+    pf <- parent.frame()
+    eval.weights <- try(eval(weights, envir=df, enclos=pf))
+    if(inherits(eval.weights, "try-error"))
+      stop("Unable to evaluate expression for weights", call.=FALSE)
+    if(!check.nvector(eval.weights, nsegments(X), fatal=FALSE, warn=TRUE))
+      stop("Result of evaluating the expression for weights has wrong format")
+    weights <- eval.weights
+  } else stop("Unrecognised format for argument 'weights'")
+  ##
+  if(is.multitype(X)) {
+    mks <- marks(X)
+    answer <- as.vector(tapply(weights * len, mks, sum))/a
+    answer[is.na(answer)] <- 0
+    names(answer) <- levels(mks)
+  } else {
+    answer <- sum(weights)/a
+  }
+  return(answer)
+}
+
