@@ -34,7 +34,9 @@ local({
 #
 # Also test whether minnndist(X) == min(nndist(X))
 #
-#   $Revision: 1.17 $  $Date: 2016/11/29 06:25:15 $
+# Also test nnorient()
+#
+#   $Revision: 1.20 $  $Date: 2018/10/02 01:37:12 $
 #
 
 require(spatstat)
@@ -209,6 +211,17 @@ local({
   if(any(a != b)) stop("Incorrect result for has.close(X, r, Y, periodic=TRUE)")
 })
 
+local({
+  b <- bdist.pixels(letterR, style="coords")
+})
+
+local({
+  #' test nnorient
+  nnorient(cells, domain=erosion(Window(cells), 0.1))
+  #' degenerate case
+  X <- cells[nndist(cells) > bdist.points(cells)]
+  f <- nnorient(X)
+})
 ## 
 ##    tests/percy.R
 ##
@@ -241,7 +254,7 @@ local({
 ## tests/pixelgripes.R
 ##     Problems related to pixellation of windows
 ##
-## $Revision: 1.3 $ $Date: 2015/12/29 08:54:49 $
+## $Revision: 1.4 $ $Date: 2018/10/10 08:04:10 $
 
 require(spatstat)
 local({
@@ -260,15 +273,28 @@ local({
   fit1 <- kppm(X ~ distorigin, clusters="MatClust")
   Y1 <- simulate(fit1, retry=0)
 })
+
+local({
+  ## pixellate.ppp includes mapping from (x,y) to (row, col)
+  Z <- pixellate(cells, savemap=TRUE)
+  ind <- attr(Z, "map")
+  m <- (as.matrix(Z))[ind]
+  if(!all(m == 1)) stop("Coordinate mismatch in pixellate.ppp")
+})
+
 ## 
 ## tests/polygons.R
 ##
-##  $Revision: 1.2 $ $Date: 2015/12/29 08:54:49 $
+##  $Revision: 1.3 $ $Date: 2018/07/22 02:10:07 $
 ##
 require(spatstat)
 local({
   co <- as.ppp(corners(letterR), letterR, check=FALSE)
   co[letterR]
+
+  b <- letterR$bdry
+  a <- sapply(b, xypolyselfint, yesorno=TRUE)
+  a <- lapply(b, xypolyselfint, proper=TRUE)
 })
 
 # 
@@ -445,15 +471,33 @@ local({
   sl2fit <- update(slfit, ~grad + I(grad^2))
   slfitup <- update(slfit, use.internal=TRUE)
   sl2fitup <- update(sl2fit, use.internal=TRUE)
+
+  ## (4) anova.ppm
+  fut1  <- ppm(cells ~ 1, Strauss(0.1))
+  futx  <- ppm(cells ~ x, Strauss(0.1))
+  anova(fut1, test="Chi")
+  anova(futx, test="Chi")
+  fut1a <- ppm(cells ~ 1, Strauss(0.1), rbord=0)
+  anova(fut1a, futx, test="Chi")
+  fut1d <- ppm(cells ~ 1, Strauss(0.1), nd=23)
+  anova(fut1d, futx, test="Chi")
+  ## The following doesn't work yet
+  ## futxyg <- ppm(cells ~ x + s(y), Strauss(0.1), use.gam=TRUE)
+  ## anova(futx, futxyg)
+  fatP <- ppm(amacrine ~ marks)
+  fatM <- ppm(amacrine ~ marks, MultiStrauss(matrix(0.07, 2, 2)))
+  anova(fatP, fatM, test="Chi")
 })
 
-#
+grep#
 #   tests/ppmtricks.R
 #
 #   Test backdoor exits and hidden options in ppm
 #        and summary.ppm, print.summary.ppm
 #
-#   $Revision: 1.5 $  $Date: 2015/12/29 08:54:49 $
+#   Plus assorted tricks
+#
+#   $Revision: 1.9 $  $Date: 2018/05/11 02:19:32 $
 #
 require(spatstat)
 local({
@@ -481,36 +525,157 @@ local({
   mungp   <- profilepl(ss, StraussHard, trend=~dM, Q=cats)
 
   ## (4) splitting large quadschemes into blocks
-  op <- spatstat.options(maxmatrix=5000)
+  mop <- spatstat.options(maxmatrix=5000)
   pr <- predict(ppm(cells ~ x, AreaInter(0.05)))
-
+  spatstat.options(mop)
+  
   ## (5) shortcuts in summary.ppm
   ## and corresponding behaviour of print.summary.ppm
   print(summary(fit, quick=TRUE))
   print(summary(fit, quick="entries"))
   print(summary(fit, quick="no prediction"))
   print(summary(fit, quick="no variances"))
-  
+
+  ## (6) suffstat.R
+  fitP <- update(fit, Poisson())
+  suffstat.poisson(fitP, cells)
+  fit0 <- killinteraction(fit)
+  suffstat.poisson(fit0, cells)
+
+  ## (7) support for class ppm
+  Z <- as.im(function(x,y){x}, Window(cells))
+  fitZ <- ppm(cells ~ Z)
+  U <- getppmOriginalCovariates(fitZ)
+  logLik(fitZ, absolute=TRUE)
+  fitNot <- ppm(redwood ~1, Strauss(0.1))
+  fitFast <- emend(fitNot, trace=TRUE)
+  op <- spatstat.options(project.fast=TRUE)
+  fitFast <- emend(fitNot, trace=TRUE)
   spatstat.options(op)
+  
+  fut <- kppm(redwood ~ x)
+  A <- quad.ppm(fut)
+
+  ## (8) support for class profilepl
+  rr <- data.frame(r=seq(0.05, 0.15, by=0.02))
+  ps <- profilepl(rr, Strauss, cells)
+  plot(ps)
+  simulate(ps, nrep=1e4)
+  parameters(ps)
+  fitin(ps)
+  predict(ps, type="cif")
 })
 
+reset.spatstat.options()
+#'
+#'   tests/ppp.R
+#'
+#'   $Revision: 1.4 $ $Date: 2018/11/05 00:50:01 $
+#'
+#'  Untested cases in ppp() or associated code
+
+require(spatstat)
+local({
+  X <- runifpoint(10, letterR)
+  Y <- runifpoint(3, complement.owin(letterR))
+
+  #' test handling of points out-of-bounds
+  df <- rbind(as.data.frame(X), as.data.frame(Y))
+  A <- ppp(df$x, df$y, window=letterR, marks=1:13)
+  #' test handling of points with bad coordinates
+  B <- ppp(X$x, c(X$y[1:7], c(Inf, NA, NaN)), window=letterR, marks=1:10)
+  D <- ppp(X$x, c(X$y[1:7], c(Inf, NA, NaN)), window=letterR,
+           marks=data.frame(id=1:10, u=runif(10)))
+
+  #' test print/summary/plot methods on these bad objects
+  print(A)
+  print(B)
+  print(D)
+  summary(A)
+  summary(B)
+  summary(D)
+  plot(A)
+  plot(B)
+  plot(D)
+  
+  #' subset operator --- cases not covered elsewhere
+  #'   subset index is a logical image
+  Z <- distmap(letterR, invert=TRUE)
+  V <- (Z > 0.2)
+  XV <- X[V]
+  #'   multiple columns of marks
+  fun3 <- finpines[1:3]
+  #'   multiple columns of marks, one of which is a factor
+  U <- finpines
+  marks(U)[,2] <- factor(c(rep("A", 60), rep("B", npoints(U)-60)))
+  UU <- U[1:3, drop=TRUE]
+
+  #' test as.ppp for spatial package if it is not installed
+  FR <- Frame(letterR)
+  as.ppp(list(x=X$x, y=X$y,
+              xl=FR$xrange[1], xu=FR$xrange[2],
+              yl=FR$yrange[1], yu=FR$yrange[2]))
+
+  #' various utilities
+  periodify(cells, 2)
+  periodify(demopat, 2)
+
+  #'
+  a <- multiplicity(finpines)
+  a <- multiplicity(longleaf)
+
+  ## superimpose.ppp, extra cases
+  X <- runifpoint(20)
+  A <- superimpose(cells, X, W="convex")
+  A <- superimpose(cells, X, W=ripras)
+  ## superimpose.splitppp
+  Y <- superimpose(split(amacrine))
+})
 #
 # tests/ppx.R
 #
 # Test operations for ppx objects
 #
-#  $Revision: 1.3 $ $Date: 2017/03/02 01:19:13 $
+#  $Revision: 1.5 $ $Date: 2019/01/02 07:58:20 $
 #
 
 require(spatstat)
 
 local({
-  df <- data.frame(x=c(1,2,2,1), y=c(1,2,3,1), z=c(2,3,4,2))
+  #' general tests
+  df <- data.frame(x=c(1,2,2,1)/4, y=c(1,2,3,1)/4, z=c(2,3,4,3)/5)
   X <- ppx(data=df, coord.type=rep("s", 3), domain=box3())
   unique(X)
   duplicated(X)
+  anyDuplicated(X)
   multiplicity(X)
+  print(X)
+  summary(X)
+  plot(X)
+  domain(X)
+  unitname(X) <- c("metre", "metres")
+  unitname(X)
 
+  #' subset operator
+  X[integer(0)]
+  Y <- X %mark% data.frame(a=df$x, b=1:4)
+  Y[1:2]
+  Y[FALSE]
+  marks(Y) <- as.data.frame(marks(Y))
+  Y[integer(0)]
+  Y[1:2]
+  Y[FALSE]
+
+  #' two dimensional
+  A <- ppx(data=df[,1:2], coord.type=rep("s", 2), domain=square(1))
+  plot(A)
+  B <- ppx(data=df[,1:2], coord.type=rep("s", 2), domain=NULL)
+  plot(B)
+  #' one dimensional
+  E <- ppx(data=data.frame(x=runif(10)))
+  plot(E)
+  
+  #' bug
   stopifnot(identical(unmark(chicago[1]),
                       unmark(chicago)[1]))
 
@@ -518,13 +683,29 @@ local({
   U <- chicago[integer(0)]
   V <- U %mark% 1
   V <- U %mark% factor("a")
+
+  #' simplify lower-dimensional patterns
+  X3 <- ppx(data=df, coord.type=rep("s", 3), domain=box3(), simplify=TRUE)
+  stopifnot(is.pp3(X3))
+  X2 <- ppx(data=df[,1:2], coord.type=rep("s", 2), domain=square(1), simplify=TRUE)
+  stopifnot(is.ppp(X2))
+
+  #' marks<-.ppx
+  M <- as.matrix(X)
+  marks(X) <- df[,1]
+  marks(X) <- df[,integer(0)]
+
+  #' trivial cases of random generators
+  B4 <- boxx(0:1, 0:1, 0:1, 0:1)
+  Z0 <- runifpointx(0, domain=B4, nsim=2)
+  Z1 <- runifpointx(1, domain=B4, nsim=2)
 })
 #
 # tests/prediction.R
 #
 # Things that might go wrong with predict()
 #
-#  $Revision: 1.4 $ $Date: 2016/03/04 03:14:40 $
+#  $Revision: 1.8 $ $Date: 2018/11/01 13:20:23 $
 #
 
 require(spatstat)
@@ -563,7 +744,35 @@ local({
   a <- relrisk(fut, control=2, relative=TRUE)
   a <- relrisk(fut, se=TRUE)
   a <- relrisk(fut, relative=TRUE, se=TRUE)
-  
+
+  ## untested cases of predict.ppm
+  fit0 <- ppm(cells)
+  a <- predict(fit0, interval="confidence")
+  a <- predict(fit0, interval="confidence", type="count")
+  fit  <- ppm(cells ~ x)
+  b <- predict(fit, type="count",                            se=TRUE)
+  b <- predict(fit, type="count", window=square(0.5),        se=TRUE)
+  b <- predict(fit, type="count", window=quadrats(cells, 3), se=TRUE)
+  ## superseded usages
+  b <- predict(fit, type="se", getoutofjail=TRUE)
+  b <- predict(fit, total=TRUE)
+  b <- predict(fit, total=square(0.5))
+  b <- predict(fit, total=quadrats(cells, 3))
+
+  ## supporting code
+  u <- model.se.image(fit, square(0.5))
+  u <- model.se.image(fit, square(0.5), what="cv")
+  u <- model.se.image(fit, square(0.5), what="ce")
+
+  ##
+  fut <- ppm(cells ~ x, Strauss(0.1))
+  df <- data.frame(x=runif(10), y=runif(10),
+                   Interaction=sample(0:1, 10, TRUE))
+  m10 <- PPMmodelmatrix(fut, data=df)
+  mmm <- PPMmodelmatrix(fut, Q=quad.ppm(fut))
+  fut1 <- ppm(cells ~ 1, Strauss(0.1))
+  effectfun(fut1, "x")
+  effectfun(fut1, "y")
 })
 
 #
@@ -591,7 +800,13 @@ local({
   fit3 <- ppm(amacrine ~1, MultiStraussHard(types=c("off", "on"),
                                             iradii=r, hradii=r/5))
   chk(emend(fit3))
-  
+
+  #' code coverage
+  op <- spatstat.options(project.fast=TRUE)
+  fut <- emend(fit, trace=TRUE)
+  chk(fut)
+  spatstat.options(op)
+
   # hierarchical
   ra <- r
   r[2,1] <- NA
@@ -622,3 +837,5 @@ local({
   chk(f7)
 
 })
+
+reset.spatstat.options()
